#!/bin/bash -u
# SAVE_FILE_HOOK=cp $SELF ~/.VirtualBox/gemeinsam/Strap/; cp $SELF /media/sf_gemeinsam/pub/

javaCommand=$(which java 2>/dev/null)
#########################################################################
# Your Java installation was not recognized ...
# You need to tell the system the full path of java
# For this purpose enter a line like  javaCommand=/here/the/path/jre../bin/java
#########################################################################
set -o pipefail
[[ -x $0 ]]||chmod +x $0
# Note: Not using .zip suffix in variable JREZIP to prevent Safari from auto-extracting
##########################
### Begin Detect Proxy ###
##########################
readonly URLSTRAP=http://www.bioinformatics.org/strap
readonly DETECTPROXY_URL=$URLSTRAP/Binaries/
readonly DETECTPROXY_PATTERN=linuxAMD64
[[ -f /mnt/tmpfs/meine_IP_Nummer.txt ]] && readonly IS_MY_PC=1 || IS_MY_PC=0
##########################################################################################################
### Wget Downloads the file of the URL into current directory                                          ###
### Proxy Javascript code ending with  .pac or .dat are obtainted without proxy and writtent to stdout ###
##########################################################################################################
Wget(){
    local noproxy=0
    local dl=1
    local url=$1
    local fn=${url##*/}
    local err=1
    [[ $url == *.pac || $url == *.dat ]] && dl=0 && noproxy=1
    [[ $url == *probing_proxy ]] && dl=0
    mkdir -p tmp
    pushd tmp
    (($dl))&&rm -f $fn
    if ((MACOS));then
        local opt=
        (($dl)) && opt=$opt' -o '$fn
        (($noproxy))&&opt='--noproxy '
        echo -e -n "\n export http_proxy=$http_proxy; curl -v $opt $url " >/dev/stderr
        curl --fail $opt $url && err=0
    else
        local opt='-O - '
        (($dl))&&opt=' -N '
        (($noproxy))&&opt=$opt' --no-proxy '
        echo -e -n "\n export http_proxy=$http_proxy; wget $opt $url " >/dev/stderr
        wget $opt  $url 2>/dev/null && err=0

    fi
    if (($err));then
        echo ' '$RED_FAILED
    else
        echo ' '$GREEN_SUCCESS >/dev/stderr
        (($dl))&&mv -f $fn ..
    fi
    popd
    return $err
}
##########################
### Begin Detect Proxy ###
##########################
_detectProxy_check(){
    local save=${http_proxy:-}
    export http_proxy=$1
    local method=${2:-}
    local ret=999
    # fgrep -q not working!!
    if Wget $DETECTPROXY_URL'?probing_proxy' | fgrep -i $DETECTPROXY_PATTERN>/dev/null; then
        echo $ANSI_GREEN Success $ANSI_RESET
        _detectProxyResult=1
        ret=0
    fi
    echo -n -e '\n checkProxy  '$http_proxy' obtained by '$method' '
    if (($ret==0));then
        echo $ANSI_GREEN Success $ANSI_RESET
        return
    fi
    echo $ANSI_RED Failed $ANSI_RESET
    export http_proxy=$save
    return 1
}
_detectProxyResult=0
detectProxy(){
    (($_detectProxyResult))&&return
    ### Encoded in script file name
    local s=' '${BASH_SOURCE[0]##*/}
    s=${s%.sh}' '
    local host=${s##* PH=}
    if [[ $s != $host ]];then
        local host=${host%% *}
        if [[ ! -s $host ]]; then
            local port=${s##* PP=}
            if [[ $s != $port ]]; then
                port=${port%% *}
            else
                port=8080
            fi
            _detectProxy_check $host:$port ENCODED-IN-SRIPT-NAME && return
        fi
    fi
    ### environment-variable
    [[ ! -z ${http_proxy:-} ]] && _detectProxy_check $http_proxy ENVIRONMENT-VARIABLE && return
    ### Direct
    _detectProxy_check '' DIRECT-NO-PROXY && return
    ### System
    if ((MACOS));then
        local method=SCUTIL
        local proxy=$(scutil --proxy|awk '/HTTPEnable/ {enabled=$3;} /HTTPProxy/ {s=$3;} /HTTPPort/ {p=$3;} END { if (enabled=="1") print s":"p;}')
    elif which gsettings >/dev/null; then
        local method=GSETTINGS
        if which gsettings >/dev/null; then
            local host=$(gsettings get org.gnome.system.proxy.http host|sed "s|'||g")
            local port=$(gsettings get org.gnome.system.proxy.http port|sed "s|'||g")
            [[ ! -z $host ]] && [[ ! -z $port ]] && export http_proxy=${host#http://}:$port
            local proxy=$http_proxy
        fi
    fi
    [[ ! -z ${proxy:-} ]] &&  _detectProxy_check $proxy $method && return
    ### PAC-file
    local pp=$((Wget http://proxy/proxy.pac; Wget http://wpad/wpad.dat) 2>/dev/null|tr "'" ' '|sed 's|[ "]|\n|g' | grep '[a-zA-Z0-9.]:[0-9]\+' | sort -u)
    for p in $pp; do
        _detectProxy_check $p PAC-FILE && return
    done
    _detectProxyResult=-1
    cat<<EOF
$ANSI_RED Failed reach site $ANSI_RESET $DETECTPROXY_URL.
A possible reason is that the proxy settings not correctly identified
Consider inserting a line at the top of this this script of the form
             export http_proxy=my-proxy:8080
EOF
    return 1
}
#########################
### End Detect Proxy ###
#########################
checkZip() {
    local z=${1:-$JREZIP}
    [[ -f $z ]]||return 1
    if ((MACOS)); then
        unzip -t -q $z 2>/dev/null >/dev/null
    else
        tar tzf $z 2>/dev/null >/dev/null
    fi
    if (($?)); then
        echo -n $ANSI_RED Archive is corrupt.$ANSI_RESET' '
        mv -f $z $z.corrupt
        ls -l -h $z.corrupt
        echo $ANSI_RESET
        return 1
    else
        echo -n $ANSI_GREEN Archive is OK$ANSI_RESET' '
        ls -l -h $z
        return 0
    fi
}
####################
### Print banner ###
####################
gradient(){
    local L=${#1}
    local b=0
    local r=5
    local g=1
    for((i=0;i<L;i++));do
        if ((i%6==0));then
            ((--r<0))&&r=0
        elif ((i%6==4));then
            ((++b>5))&&b=5&&g=0
        fi
        echo -n $'\e[38;5;'$((16+36*r+6*g+b))'m'"${1:$i:1}"
    done
    echo
}
extractJRE(){
    echo $ANSI_MAGENTA extractJRE $ANSI_RESET
    [[ -f $JREZIP ]] || return 1
    echo $ANSI_MAGENTA Extracting $JREZIP to $JREDIR $ANSI_RESET
    pushd $dotsa/tmp
    if ((MACOS));then
        unzip $JREZIP
    else
        tar vxzf $JREZIP
    fi
    if [[ $? != 0 ]] || ! ${JREDIR##*/}/bin/java -version; then
        echo $RED_ERROR Extracted Java not working: ${JREDIR##*/}/bin/java -version
        exit 1
    fi
    mv ${JREDIR##*/} $dotsa
    popd
    rmdir $JREDIR.tmp
    ls -l $JREDIR
}
##############################################
### If Strap starts, then exit afterwards  ###
##############################################
launchStrap(){
    ((simulateNotStartingStrap)) && return
    local proxyJava=
    if [[ ! -z ${http_proxy:-} ]]; then
        proxyJava='-Dhttp.proxyHost='${http_proxy%:*}' -Dhttps.proxyPort='${http_proxy#*:}
        echo $ANSI_GREEN Detected system web proxyJava settings: $proxyJava $ANSI_RESET
    fi
    for excludeVersions in 567 56; do # Do not like Version jre 1.6
        for j in $javaCommand $JREDIR/bin/java /Library/Java/JavaVirtualMachines/*/Contents/Home/bin/java '/Library/Internet Plug-Ins/JavaAppletPlugin.plugin/Contents/Home/bin/java'; do
            ((MACOS==0)) && [[ $javaCommand == /Library/* ]] && continue
            ((simulateJavaNotInstalled)) && [[ $j != $JREDIR/bin/java ]] && continue
            if [[ -f "$j" ]]; then
                ((MACOS)) && fgrep -q -i 'No Java runtime present, requesting install.' "$j" && continue
                v=$("$j" -version 2>&1 | head -n 1)
                if [[ $? == 0 ]]; then
                    echo  '"'$j'" '  $v
                    if  echo $v|grep 'java version "1\.['$excludeVersions']\.'; then
                        echo -e '\b\b   Wrong version '$v
                        continue
                    else
                        echo -e '\b\b   ' $ANSI_GREEN  $v $ANSI_RESET
                    fi
                    echo
                    set -x
                    "$j" -version 2>/dev/null && "$j" $jreArgs -cp $CP:$jar $proxyJava charite.christo.strap.Strap -updateStrapJar=$dotsa/new_strap.jar -stdout -stderr -probeWebProxy -callingBatch="$0" "$@"
                    set +x
                    makeMenuEntry
                    linuxRegisterJavaws
                    echo
                    exit
                fi
            fi
        done
    done
}
######################################
### End function decalarations.    ###
### Main                           ###
######################################

readonly ANSI_RED=$'\e[41m'
readonly ANSI_MAGENTA=$'\e[45m'
readonly ANSI_RESET=$'\e[0m'
readonly ANSI_GREEN=$'\e[42m'
readonly ANSI_FG_GREEN=$'\e[32m'
readonly ANSI_FG_RED=$'\e[31m'
readonly ANSI_FG_MAGENTA=$'\e[35m'
readonly ANSI_INVERSE=$'\e[7m'
readonly GREEN_SUCCESS=$'\e[32m Success \e[0m'
readonly RED_FAILED=$'\e[31m Failed \e[0m'
readonly URL_JAVA=https://www.java.com/en/download/
readonly dotsa=~/.StrapAlign
readonly STRAP_PNG=$dotsa/Strap.png
readonly JAVA_PNG=$dotsa/Java.png
readonly jarFromZip=${BASH_SOURCE[0]%/*}/strap.jar
readonly jar=$dotsa/strap.jar
readonly xdg=$dotsa/xdg
readonly xdg_strap=$xdg/sequence_alignments_strap.desktop
readonly xdg_javaws=$xdg/javaws.desktop
readonly xdg_javaws_settings=$xdg/javaws_settings.desktop
readonly STRAP_SH=$dotsa/strap.sh
diff -q ${BASH_SOURCE[0]} $STRAP_SH
readonly SCRIPT_CHANGED=$?
gradient ' ____    __'
gradient '/\  _`\ /\ \__'
gradient '\ \,\L\_\ \ ,_\  _ __    __     _____'
gradient ' \/_\__ \\ \ \/ /\` __\/ __`\  /\  __`\'
gradient '   /\ \L\ \ \ \_\ \ \//\ \L\.\_\ \ \L\ \'
gradient '   \ `\____\ \__\\ \_\\ \__/.\_\\ \ ,__/'
gradient '    \/_____/\/__/ \/_/ \/__/\/_/ \ \ \/'
gradient '                                  \ \_\'
gradient '                                   \/_/'
echo $ANSI_RESET$ANSI_INVERSE This script starts  $jar $ANSI_RESET
echo Type $0 -h for help on program parameters
echo
mkdir -p $dotsa/tmp
# *************************
# *** Mac or Linux/Unix ***
# *************************
readonly UNAME_M=$(uname -m)
if [[ $(uname) == Darwin ]]; then
    # Not using .zip suffix in variable JREZIP to prevent Safari from auto-extracting
    readonly MACOS=1
    readonly JREZIP=$dotsa/jre-8u201_macosx.zipfile
    readonly JREDIR=$dotsa/jre-8u201
    readonly JREURL=$URLSTRAP/Binaries/macIntel/${JREZIP##*/}
    readonly DOWNLOADED=$HOME/Downloads/${JREZIP##*/}
else
    readonly MACOS=0
    readonly JREDIR=$dotsa/jre1.8.0_201
    if [[ ${UNAME_M/64} != $UNAME_M ]]; then
        #Running java32 on Linux64 is not possible unless   apt-get install libc6-i386
        readonly JREZIP=$dotsa/jre-8u201-linux-x64.tar.gz
        readonly JREURL=$URLSTRAP/Binaries/linuxAMD64/${JREZIP##*/}
    else
        readonly JREZIP=$dotsa/jre-8u201-linux-i586.tar.gz
        readonly JREURL=$URLSTRAP/Binaries/linux386/${JREZIP##*/}
    fi
fi
# ****************************
# *** Command line options ***
# ****************************
simulateJavaNotInstalled=0
simulateNotStartingStrap=0
simulateOutOfDate=0
while getopts EPJS opt; do
	case $opt in
        J) echo rm -r -f ${DOWNLOADED:-} $JREZIP $JREDIR
           simulateJavaNotInstalled=1;;
        S) simulateNotStartingStrap=1;;
        P) detectProxy; exit;;
        E) simulateOutOfDate=1;;
	esac
done
# **********************
# *** JVM PARAMETERS ***
# **********************
CP=
jreArgs=
for i in "$@"; do
    [[ $i != ${i#JVM_} ]] && jreArgs=$jreArgs' '${i#JVM_}
    [[ $i != ${i#-X} ]] && jreArgs=$jreArgs' '$i
    [[ $i != ${i#-D} ]] && jreArgs=$jreArgs' '$i
    [[ $i != ${i#-cp=} ]] && CP=${i#-cp=}':'$CP
done
# ********************
# *** AUTO UPDATES ***
# ********************
# In the last strap session, an update new_strap.jar might have been downloaded.
for update in $jarFromZip $dotsa/new_strap.jar; do
    if  [[ -s $update ]] && ! [[ $update == $jar ]]; then
        if [[ -f $update.keep || ! -w $update ]]; then
            readonly updateSeconds=0$(unzip -p  $update S.rsc)
            readonly jarSeconds=0$(unzip -p  $jar S.rsc)
            # For testing only
            if (($updateSeconds>$jarSeconds)); then
                echo Copying $update to $jar because $updateSeconds '>' $jarSeconds
                cp $update $jar
            else
                echo Not copying $update to $jar because $updateSeconds '<=' $jarSeconds
            fi
        else
            echo Moving $update to $jar
            mv $update $jar
            # There is a problem with the time+date of the file in the zip.
            [[ $update == $jarFromZip ]] && touch $jar
        fi
    fi
done
if (($simulateOutOfDate)); then
    touch -d '5 years ago' $dotsa/strap.jar
    echo -n $ANSI_MAGENTA simulateOutOfDate' '
    ls -l $dotsa/strap.jar
    echo $ANSI_RESET
fi
launchStrap
echo $ANSI_MAGENTA No Java installation found on your system. Now installing  $JREDIR $ANSI_RESET
echo 'If a Java 1.7 installation somewhere on the hard-disk has been overlooked'
echo 'then edit this script and follow the instructions at the top of the script.'
echo
read -n1 -r -p 'Press any key to continue Java installation ...' key
[[ ${UNAME_M/86} == $UNAME_M ]] && read -n1 -r -p 'It seems that your CPU is not Intel/AMD x86 compatible. Continue anyway?...' key
detectProxy
if ! checkZip; then
    echo $ANSI_MAGENTA Downloading $JREZIP '(80 MB) to' $HOME/.StrapAlign/ $ANSI_RESET
    cd $dotsa
    Wget $JREURL
    echo -n After Wget; ls -l $JREZIP
fi
if checkZip; then
    extractJRE
    launchStrap
fi
if ((MACOS)); then
    if [[ ! -f $DOWNLOADED ]]; then
        echo $ANSI_MAGENTA Downloading $JREURL using Safari $ANSI_RESET
        open -a Safari $JREURL  --args h=222 w=444
        while [[ ! -f $DOWNLOADED ]]; do
            echo -n Waiting for $DOWNLOADED ' '
            ls -l $DOWNLOADED*
            sleep 3
        done
    fi
    mv $DOWNLOADED $JREZIP
    extractJRE
    launchStrap
fi
echo $ANSI_RED Download Java failed $JREURL. Download it manually and extract to $dotsa $ANSI_RESET
((MACOS==0)) && ! [[ -f $jarFromZip.keep ]] && read -n1 -r -p 'Press any key to quit ...' key
